import pandas as pd
from pymongo import MongoClient
import pickle
from flask import Flask, request
import time
import logging
import traceback
from common.database_dml import get_data_from_mongo, insert_data_into_mongo


app = Flask('model_prediction_photovoltaic_physical——service')


def str_to_list(arg):
    if arg == '':
        return []
    else:
        return arg.split(',')


def forecast_data_distribution(pre_data, args):
    col_time = args['col_time']
    farm_id = args['farm_id']
    col_radiance = args['col_radiance']
    radiance_max = float(args['radiance_max'])
    cap = float(args['cap'])
    pre_data['farm_id'] = farm_id
    pre_data['power_forecast'] = round(pre_data[col_radiance] * cap / radiance_max, 2)
    if 'sunrise_time' in args:
        sunrise_time = args['sunrise_time']
        pre_data.loc[pre_data[col_time].dt.time < sunrise_time, 'power_forecast'] = 0
    if 'sunset_time' in args:
        sunset_time = args['sunset_time']
        pre_data[pre_data[col_time] > sunset_time, 'power_forecast'] = 0
    return pre_data[['farm_id', 'date_time', 'power_forecast']]


def model_prediction(df, args):
    # 新增日出、日落时间参数
    howLongAgo, farm_id, target, cap, col_radiance, radiance_max, model_name, col_time = int(args['howLongAgo']), args['farm_id'], \
    args['target'], args['cap'], args['col_radiance'], args['radiance_max'], args['model_name'], args['col_time']
    df['power_forecast'] = round(df[col_radiance]*cap/radiance_max, 2)
    df.loc[df['power_forecast'] < 0, 'power_forecast'] = 0
    if 'sunrise_time' in args:
        sunrise_time = args['sunrise_time']
        df.loc[df[col_time].dt.time < sunrise_time, 'power_forecast'] = 0
    if 'sunset_time' in args:
        sunset_time = args['sunset_time']
        df[df[col_time] > sunset_time, 'power_forecast'] = 0
    df['model'] = model_name
    df['howLongAgo'] = howLongAgo
    df['farm_id'] = farm_id
    print("model predict result  successfully!")
    return df[['dateTime', 'howLongAgo', 'model', 'farm_id', 'power_forecast', target]]


@app.route('/model_prediction_photovoltaic_physical', methods=['POST'])
def model_prediction_photovoltaic_physical():
    # 获取程序开始时间
    start_time = time.time()
    result = {}
    success = 0
    print("Program starts execution!")
    try:
        args = request.values.to_dict()
        print('args', args)
        logger.info(args)
        forecast_file = int(args['forecast_file'])
        power_df = get_data_from_mongo(args)
        if forecast_file == 1:
            predict_data = forecast_data_distribution(power_df, args)
        else:
            predict_data = model_prediction(power_df, args)
        insert_data_into_mongo(predict_data, args)
        success = 1
    except Exception as e:
        my_exception = traceback.format_exc()
        my_exception.replace("\n", "\t")
        result['msg'] = my_exception
    end_time = time.time()
    result['success'] = success
    result['args'] = args
    result['start_time'] = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(start_time))
    result['end_time'] = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(end_time))
    print("Program execution ends!")
    return result


if __name__ == "__main__":
    print("Program starts execution!")
    logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    logger = logging.getLogger("model_prediction_photovoltaic_physical log")
    from waitress import serve

    serve(app, host="0.0.0.0", port=10126)
    print("server start!")
